"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const error_1 = require("@pnpm/error");
const can_write_to_dir_1 = require("can-write-to-dir");
const path = require("path");
const PATH = require("path-name");
function default_1() {
    var _a, _b;
    if (!process.env[PATH]) {
        throw new error_1.default('NO_PATH_ENV', `Couldn't find a global directory for executables because the "${PATH}" environment variable is not set.`);
    }
    const dirs = (_b = (_a = process.env[PATH]) === null || _a === void 0 ? void 0 : _a.split(path.delimiter)) !== null && _b !== void 0 ? _b : [];
    return pickBestGlobalBinDir(dirs);
}
exports.default = default_1;
function pickBestGlobalBinDir(dirs) {
    const nodeBinDir = path.dirname(process.execPath);
    const noWriteAccessDirs = [];
    for (const dir of dirs) {
        if (isUnderDir('node', dir) ||
            isUnderDir('nodejs', dir) ||
            isUnderDir('npm', dir) ||
            isUnderDir('pnpm', dir) ||
            nodeBinDir === dir) {
            if (canWriteToDirAndExists(dir))
                return dir;
            noWriteAccessDirs.push(dir);
        }
    }
    if (noWriteAccessDirs.length === 0) {
        throw new error_1.default('NO_GLOBAL_BIN_DIR', "Couldn't find a suitable global executables directory.", {
            hint: `There should be a node, nodejs, npm, or pnpm directory in the "${PATH}" environment variable`,
        });
    }
    throw new error_1.default('GLOBAL_BIN_DIR_PERMISSION', 'No write access to the found global executable directories', {
        hint: `The found directories:
${noWriteAccessDirs.join('\n')}`,
    });
}
function isUnderDir(dir, target) {
    target = target.endsWith(path.sep) ? target : `${target}${path.sep}`;
    return target.includes(`${path.sep}${dir}${path.sep}`) ||
        target.includes(`${path.sep}.${dir}${path.sep}`);
}
function canWriteToDirAndExists(dir) {
    try {
        return can_write_to_dir_1.sync(dir);
    }
    catch (err) {
        if (err.code !== 'ENOENT')
            throw err;
        return false;
    }
}
//# sourceMappingURL=index.js.map